#INCLUDE "class.CH"
#INCLUDE "dcdialog.CH"

//
// The class DC_WebSocketClient serves as client
// side of the DC_WebSocketHandler
//
CLASS DC_WebSocketClient FROM WebSocketClient, DC_WebSocketData
  EXPORTED
    VAR response
    VAR serverTimeout
    VAR hostName
    VAR hostPort
    VAR clientName
    VAR serverName
    VAR isConnected
    METHOD onText
    METHOD onConnect
    METHOD onDisconnect
    METHOD onBinary
    METHOD execute
    METHOD init
    METHOD destroy
    METHOD keepAlive
ENDCLASS

METHOD DC_WebSocketClient:Init( cHostName, cServerName, nHostPort, cClientName )

LOCAL oClient

cHostName := Strtran(cHostName,'http://','')
cHostName := Strtran(cHostName,'Http://','')
cHostName := Strtran(cHostName,'HTTP://','')

::isConnected := .f.
::hostName := cHostName
::hostPort := nHostPort
::clientName := cClientName
::serverName := cServerName
::serverTimeOut := 20
DEFAULT ::serverName := ::hostName
::dc_websocketdata:init()

// wtf 'SocketConnect' COLOR GRA_CLR_DARKCYAN WINDOW 1

oClient := ::webSocketClient:init(cHostName,cServerName,nHostPort,cClientName)

::isConnected := oClient:connect()

IF .NOT. ::isConnected
  DCMSGBOX "Connection failed to server " + ::hostName + ':' + Alltrim(Str(::hostPort)) ;
     FONT '12.Lucida Console'
ENDIF

RETURN oClient

* ---------

METHOD DC_WebSocketClient:onText( cText )
  // do nothing
RETURN self

METHOD DC_WebSocketClient:onBinary( cBinary )
  ::response := Bin2Var(cBinary)
RETURN self

METHOD DC_WebSocketClient:onConnect()
  ::isConnected := .t.
RETURN self

METHOD DC_WebSocketClient:onDisconnect()
  ::isConnected := .f.
RETURN

METHOD DC_WebSocketClient:destroy()
RETURN ::disconnect()

METHOD DC_WebSocketClient:keepAlive()
RETURN self

METHOD DC_WebSocketClient:execute( cAction )

LOCAL nSeconds, lHandled

IF Valtype(cAction) == 'C'
  ::action := cAction
ENDIF

// wtf cAction COLOR GRA_CLR_DARKPINK WINDOW 1

::response := NIL

::sendBinary( Var2Bin(self) )

nSeconds := Seconds()
DO WHILE ::isConnected() .AND. Empty(::response)
  lHandled := ::handleMessage( .F. )
  IF Seconds() - nSeconds > ::serverTimeOut
    DCMSGBOX ::serverName + ' is not responding'
    EXIT
  ENDIF
  Sleep(5)
ENDDO

RETURN ::response


// A websocket handler must implement the methods
// :onConnect(), onDisconnect(), onText() and
// :onBinary().
//
// The methods are triggered from the HttpEndpoint.
//
CLASS DC_WebSocketHandler FROM WebSocketHandler, DC_WebSocketData
  EXPORTED:
    VAR requestHandler
    VAR requestClient
    CLASS VAR clientList
    METHOD onConnect
    METHOD onDisconnect
    METHOD onText
    METHOD onBinary
    CLASS METHOD InitClass
ENDCLASS

* --------
CLASS METHOD DC_WebSocketHandler:initClass()
::clientList := {}
RETURN self

* --------

METHOD DC_WebSocketHandler:onConnect()

AAdd(::clientList,self)

RETURN self

* --------

METHOD DC_WebSocketHandler:onDisconnect()

LOCAL nFound
nFound := AScan(::clientList,self)

IF nFound > 0
  ARemove(::clientList,nFound)
ENDIF

RETURN self

* ---------

METHOD DC_WebSocketHandler:onText( cText )
::sendText( cText )
RETURN self

* ---------

METHOD DC_WebSocketHandler:onBinary( cBinary )

LOCAL i, oRequest := Bin2Var(cBinary), oResponse

IF Valtype(oRequest) == 'A'
  FOR i := 1 TO Len(oRequest)
    IF Valtype(oRequest[i]) == 'B'
      ::sendBinary( Var2Bin(Eval(oRequest[i])) )
    ENDIF
  NEXT
ENDIF

IF Valtype(::requestHandler) # 'B'
  DCMSGBOX 'No ::requestHandler codeblock defined in ' + self:ClassName() ;
   FONT '12.Lucida Console'
ELSE
  oResponse := Eval(::requestHandler,oRequest,self)
ENDIF

::sendBinary( Var2Bin(oResponse) )

RETURN self

* ==============

CLASS DC_WebSocketData

EXPORTED:

VAR varList
VAR action
VAR faultList

INLINE METHOD Init()

::varList := {}
::faultList := {}

RETURN self

* --------

INLINE METHOD setVar( cVarName, xData )

LOCAL nFound

nFound := Ascan(::varList,{|a|a[1]==Upper(cVarName)})

IF nFound == 0
  AAdd(::varList,{Upper(cVarName),xData})
ELSE
  ::varList[nFound,2] := xData
ENDIF

RETURN self

* --------

INLINE METHOD setFault( cVarName, xData )

LOCAL nFound

nFound := Ascan(::faultList,{|a|a[1]==Upper(cVarName)})

IF nFound == 0
  AAdd(::faultList,{Upper(cVarName),xData})
ELSE
  ::faultList[nFound,2] := xData
ENDIF

RETURN self


* --------

INLINE METHOD getVar( cVarName )

LOCAL nFound := Ascan(::varList,{|a|Upper(Trim(a[1]))==Upper(Trim(cVarName))})

IF nFound > 0
  RETURN ::varList[nFound,2]
ENDIF

RETURN nil

* --------

INLINE METHOD getAllVars()

RETURN ::varList

ENDCLASS



